from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..types import UNSET, Unset






T = TypeVar("T", bound="TagBreakdown")



@_attrs_define
class TagBreakdown:
    """ Spend/token breakdown keyed by tag value (e.g. wrk-xxx, usr-xxx).

        Attributes:
            spend (float | Unset):  Default: 0.0.
            total_tokens (int | Unset):  Default: 0.
            requests (int | Unset):  Default: 0.
     """

    spend: float | Unset = 0.0
    total_tokens: int | Unset = 0
    requests: int | Unset = 0
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)





    def to_dict(self) -> dict[str, Any]:
        spend = self.spend

        total_tokens = self.total_tokens

        requests = self.requests


        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
        })
        if spend is not UNSET:
            field_dict["spend"] = spend
        if total_tokens is not UNSET:
            field_dict["total_tokens"] = total_tokens
        if requests is not UNSET:
            field_dict["requests"] = requests

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        spend = d.pop("spend", UNSET)

        total_tokens = d.pop("total_tokens", UNSET)

        requests = d.pop("requests", UNSET)

        tag_breakdown = cls(
            spend=spend,
            total_tokens=total_tokens,
            requests=requests,
        )


        tag_breakdown.additional_properties = d
        return tag_breakdown

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset







T = TypeVar("T", bound="AllLlmCallsAggregate")



@_attrs_define
class AllLlmCallsAggregate:
    """ Aggregate token usage across all LLM calls within a single request.

        Attributes:
            count (int): Number of LLM calls made
            input_tokens (int): Sum of prompt tokens across all calls
            output_tokens (int): Sum of completion tokens across all calls
            reasoning_tokens (int): Sum of reasoning tokens across all calls
            total_tokens (int): Sum of all tokens across all calls
            last_input_tokens (int): Input tokens from the most recent LLM call (= current thread size)
     """

    count: int
    input_tokens: int
    output_tokens: int
    reasoning_tokens: int
    total_tokens: int
    last_input_tokens: int
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)





    def to_dict(self) -> dict[str, Any]:
        count = self.count

        input_tokens = self.input_tokens

        output_tokens = self.output_tokens

        reasoning_tokens = self.reasoning_tokens

        total_tokens = self.total_tokens

        last_input_tokens = self.last_input_tokens


        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "count": count,
            "input_tokens": input_tokens,
            "output_tokens": output_tokens,
            "reasoning_tokens": reasoning_tokens,
            "total_tokens": total_tokens,
            "last_input_tokens": last_input_tokens,
        })

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        count = d.pop("count")

        input_tokens = d.pop("input_tokens")

        output_tokens = d.pop("output_tokens")

        reasoning_tokens = d.pop("reasoning_tokens")

        total_tokens = d.pop("total_tokens")

        last_input_tokens = d.pop("last_input_tokens")

        all_llm_calls_aggregate = cls(
            count=count,
            input_tokens=input_tokens,
            output_tokens=output_tokens,
            reasoning_tokens=reasoning_tokens,
            total_tokens=total_tokens,
            last_input_tokens=last_input_tokens,
        )


        all_llm_calls_aggregate.additional_properties = d
        return all_llm_calls_aggregate

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any, cast
from urllib.parse import quote

import httpx

from ...client import AuthenticatedClient, Client
from ...types import Response, UNSET
from ... import errors

from ...models.http_validation_error import HTTPValidationError
from ...models.message_input import MessageInput
from ...models.responses_api_response import ResponsesAPIResponse
from typing import cast



def _get_kwargs(
    *,
    body: MessageInput,

) -> dict[str, Any]:
    headers: dict[str, Any] = {}


    

    

    _kwargs: dict[str, Any] = {
        "method": "post",
        "url": "/v1/responses",
    }

    _kwargs["json"] = body.to_dict()


    headers["Content-Type"] = "application/json"

    _kwargs["headers"] = headers
    return _kwargs



def _parse_response(*, client: AuthenticatedClient | Client, response: httpx.Response) -> Any | HTTPValidationError | ResponsesAPIResponse | None:
    if response.status_code == 200:
        response_200 = cast(Any, None)
        return response_200

    if response.status_code == 202:
        response_202 = ResponsesAPIResponse.from_dict(response.json())



        return response_202

    if response.status_code == 422:
        response_422 = HTTPValidationError.from_dict(response.json())



        return response_422

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(*, client: AuthenticatedClient | Client, response: httpx.Response) -> Response[Any | HTTPValidationError | ResponsesAPIResponse]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient | Client,
    body: MessageInput,

) -> Response[Any | HTTPValidationError | ResponsesAPIResponse]:
    """ Create Response

     Create a model response (OpenAI Responses API).

    Supports all combinations of ``stream`` and ``background``:
    - ``stream=false, background=false``: blocks, returns completed JSON
    - ``stream=true,  background=false``: SSE stream (resilient to disconnect)
    - ``background=true``: 202 immediately, task runs to completion

    Args:
        body (MessageInput):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Any | HTTPValidationError | ResponsesAPIResponse]
     """


    kwargs = _get_kwargs(
        body=body,

    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)

def sync(
    *,
    client: AuthenticatedClient | Client,
    body: MessageInput,

) -> Any | HTTPValidationError | ResponsesAPIResponse | None:
    """ Create Response

     Create a model response (OpenAI Responses API).

    Supports all combinations of ``stream`` and ``background``:
    - ``stream=false, background=false``: blocks, returns completed JSON
    - ``stream=true,  background=false``: SSE stream (resilient to disconnect)
    - ``background=true``: 202 immediately, task runs to completion

    Args:
        body (MessageInput):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Any | HTTPValidationError | ResponsesAPIResponse
     """


    return sync_detailed(
        client=client,
body=body,

    ).parsed

async def asyncio_detailed(
    *,
    client: AuthenticatedClient | Client,
    body: MessageInput,

) -> Response[Any | HTTPValidationError | ResponsesAPIResponse]:
    """ Create Response

     Create a model response (OpenAI Responses API).

    Supports all combinations of ``stream`` and ``background``:
    - ``stream=false, background=false``: blocks, returns completed JSON
    - ``stream=true,  background=false``: SSE stream (resilient to disconnect)
    - ``background=true``: 202 immediately, task runs to completion

    Args:
        body (MessageInput):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Any | HTTPValidationError | ResponsesAPIResponse]
     """


    kwargs = _get_kwargs(
        body=body,

    )

    response = await client.get_async_httpx_client().request(
        **kwargs
    )

    return _build_response(client=client, response=response)

async def asyncio(
    *,
    client: AuthenticatedClient | Client,
    body: MessageInput,

) -> Any | HTTPValidationError | ResponsesAPIResponse | None:
    """ Create Response

     Create a model response (OpenAI Responses API).

    Supports all combinations of ``stream`` and ``background``:
    - ``stream=false, background=false``: blocks, returns completed JSON
    - ``stream=true,  background=false``: SSE stream (resilient to disconnect)
    - ``background=true``: 202 immediately, task runs to completion

    Args:
        body (MessageInput):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Any | HTTPValidationError | ResponsesAPIResponse
     """


    return (await asyncio_detailed(
        client=client,
body=body,

    )).parsed



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..types import UNSET, Unset
from typing import cast

if TYPE_CHECKING:
  from ..models.daily_usage import DailyUsage
  from ..models.project_usage_response_by_feature import ProjectUsageResponseByFeature
  from ..models.project_usage_response_by_user import ProjectUsageResponseByUser
  from ..models.project_usage_response_by_workspace import ProjectUsageResponseByWorkspace
  from ..models.project_usage_response_model_groups import ProjectUsageResponseModelGroups





T = TypeVar("T", bound="ProjectUsageResponse")



@_attrs_define
class ProjectUsageResponse:
    """ Usage breakdown for a project's billing period.

        Attributes:
            project_ext_id (str):
            period_start (str):
            period_end (str):
            total_spend (float | Unset):  Default: 0.0.
            total_tokens (int | Unset):  Default: 0.
            total_requests (int | Unset):  Default: 0.
            daily (list[DailyUsage] | Unset):
            model_groups (ProjectUsageResponseModelGroups | Unset):
            by_workspace (ProjectUsageResponseByWorkspace | Unset):
            by_user (ProjectUsageResponseByUser | Unset):
            by_feature (ProjectUsageResponseByFeature | Unset):
            invoice_pdf (None | str | Unset):
     """

    project_ext_id: str
    period_start: str
    period_end: str
    total_spend: float | Unset = 0.0
    total_tokens: int | Unset = 0
    total_requests: int | Unset = 0
    daily: list[DailyUsage] | Unset = UNSET
    model_groups: ProjectUsageResponseModelGroups | Unset = UNSET
    by_workspace: ProjectUsageResponseByWorkspace | Unset = UNSET
    by_user: ProjectUsageResponseByUser | Unset = UNSET
    by_feature: ProjectUsageResponseByFeature | Unset = UNSET
    invoice_pdf: None | str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)





    def to_dict(self) -> dict[str, Any]:
        from ..models.project_usage_response_model_groups import ProjectUsageResponseModelGroups
        from ..models.project_usage_response_by_user import ProjectUsageResponseByUser
        from ..models.project_usage_response_by_workspace import ProjectUsageResponseByWorkspace
        from ..models.project_usage_response_by_feature import ProjectUsageResponseByFeature
        from ..models.daily_usage import DailyUsage
        project_ext_id = self.project_ext_id

        period_start = self.period_start

        period_end = self.period_end

        total_spend = self.total_spend

        total_tokens = self.total_tokens

        total_requests = self.total_requests

        daily: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.daily, Unset):
            daily = []
            for daily_item_data in self.daily:
                daily_item = daily_item_data.to_dict()
                daily.append(daily_item)



        model_groups: dict[str, Any] | Unset = UNSET
        if not isinstance(self.model_groups, Unset):
            model_groups = self.model_groups.to_dict()

        by_workspace: dict[str, Any] | Unset = UNSET
        if not isinstance(self.by_workspace, Unset):
            by_workspace = self.by_workspace.to_dict()

        by_user: dict[str, Any] | Unset = UNSET
        if not isinstance(self.by_user, Unset):
            by_user = self.by_user.to_dict()

        by_feature: dict[str, Any] | Unset = UNSET
        if not isinstance(self.by_feature, Unset):
            by_feature = self.by_feature.to_dict()

        invoice_pdf: None | str | Unset
        if isinstance(self.invoice_pdf, Unset):
            invoice_pdf = UNSET
        else:
            invoice_pdf = self.invoice_pdf


        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "project_ext_id": project_ext_id,
            "period_start": period_start,
            "period_end": period_end,
        })
        if total_spend is not UNSET:
            field_dict["total_spend"] = total_spend
        if total_tokens is not UNSET:
            field_dict["total_tokens"] = total_tokens
        if total_requests is not UNSET:
            field_dict["total_requests"] = total_requests
        if daily is not UNSET:
            field_dict["daily"] = daily
        if model_groups is not UNSET:
            field_dict["model_groups"] = model_groups
        if by_workspace is not UNSET:
            field_dict["by_workspace"] = by_workspace
        if by_user is not UNSET:
            field_dict["by_user"] = by_user
        if by_feature is not UNSET:
            field_dict["by_feature"] = by_feature
        if invoice_pdf is not UNSET:
            field_dict["invoice_pdf"] = invoice_pdf

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.daily_usage import DailyUsage
        from ..models.project_usage_response_by_feature import ProjectUsageResponseByFeature
        from ..models.project_usage_response_by_user import ProjectUsageResponseByUser
        from ..models.project_usage_response_by_workspace import ProjectUsageResponseByWorkspace
        from ..models.project_usage_response_model_groups import ProjectUsageResponseModelGroups
        d = dict(src_dict)
        project_ext_id = d.pop("project_ext_id")

        period_start = d.pop("period_start")

        period_end = d.pop("period_end")

        total_spend = d.pop("total_spend", UNSET)

        total_tokens = d.pop("total_tokens", UNSET)

        total_requests = d.pop("total_requests", UNSET)

        _daily = d.pop("daily", UNSET)
        daily: list[DailyUsage] | Unset = UNSET
        if _daily is not UNSET:
            daily = []
            for daily_item_data in _daily:
                daily_item = DailyUsage.from_dict(daily_item_data)



                daily.append(daily_item)


        _model_groups = d.pop("model_groups", UNSET)
        model_groups: ProjectUsageResponseModelGroups | Unset
        if isinstance(_model_groups,  Unset):
            model_groups = UNSET
        else:
            model_groups = ProjectUsageResponseModelGroups.from_dict(_model_groups)




        _by_workspace = d.pop("by_workspace", UNSET)
        by_workspace: ProjectUsageResponseByWorkspace | Unset
        if isinstance(_by_workspace,  Unset):
            by_workspace = UNSET
        else:
            by_workspace = ProjectUsageResponseByWorkspace.from_dict(_by_workspace)




        _by_user = d.pop("by_user", UNSET)
        by_user: ProjectUsageResponseByUser | Unset
        if isinstance(_by_user,  Unset):
            by_user = UNSET
        else:
            by_user = ProjectUsageResponseByUser.from_dict(_by_user)




        _by_feature = d.pop("by_feature", UNSET)
        by_feature: ProjectUsageResponseByFeature | Unset
        if isinstance(_by_feature,  Unset):
            by_feature = UNSET
        else:
            by_feature = ProjectUsageResponseByFeature.from_dict(_by_feature)




        def _parse_invoice_pdf(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        invoice_pdf = _parse_invoice_pdf(d.pop("invoice_pdf", UNSET))


        project_usage_response = cls(
            project_ext_id=project_ext_id,
            period_start=period_start,
            period_end=period_end,
            total_spend=total_spend,
            total_tokens=total_tokens,
            total_requests=total_requests,
            daily=daily,
            model_groups=model_groups,
            by_workspace=by_workspace,
            by_user=by_user,
            by_feature=by_feature,
            invoice_pdf=invoice_pdf,
        )


        project_usage_response.additional_properties = d
        return project_usage_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any, cast
from urllib.parse import quote

import httpx

from ...client import AuthenticatedClient, Client
from ...types import Response, UNSET
from ... import errors

from ...models.http_validation_error import HTTPValidationError
from ...models.logout_response import LogoutResponse
from ...types import UNSET, Unset
from typing import cast



def _get_kwargs(
    *,
    all_sessions: bool | Unset = False,

) -> dict[str, Any]:
    

    

    params: dict[str, Any] = {}

    params["all_sessions"] = all_sessions


    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}


    _kwargs: dict[str, Any] = {
        "method": "post",
        "url": "/v1/user/logout",
        "params": params,
    }


    return _kwargs



def _parse_response(*, client: AuthenticatedClient | Client, response: httpx.Response) -> HTTPValidationError | LogoutResponse | None:
    if response.status_code == 200:
        response_200 = LogoutResponse.from_dict(response.json())



        return response_200

    if response.status_code == 422:
        response_422 = HTTPValidationError.from_dict(response.json())



        return response_422

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(*, client: AuthenticatedClient | Client, response: httpx.Response) -> Response[HTTPValidationError | LogoutResponse]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient | Client,
    all_sessions: bool | Unset = False,

) -> Response[HTTPValidationError | LogoutResponse]:
    """ Logout

     Log out the current user. Pass ?all_sessions=true to log out from all devices.

    Args:
        all_sessions (bool | Unset): If true, log out from all sessions (all devices) Default:
            False.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[HTTPValidationError | LogoutResponse]
     """


    kwargs = _get_kwargs(
        all_sessions=all_sessions,

    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)

def sync(
    *,
    client: AuthenticatedClient | Client,
    all_sessions: bool | Unset = False,

) -> HTTPValidationError | LogoutResponse | None:
    """ Logout

     Log out the current user. Pass ?all_sessions=true to log out from all devices.

    Args:
        all_sessions (bool | Unset): If true, log out from all sessions (all devices) Default:
            False.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        HTTPValidationError | LogoutResponse
     """


    return sync_detailed(
        client=client,
all_sessions=all_sessions,

    ).parsed

async def asyncio_detailed(
    *,
    client: AuthenticatedClient | Client,
    all_sessions: bool | Unset = False,

) -> Response[HTTPValidationError | LogoutResponse]:
    """ Logout

     Log out the current user. Pass ?all_sessions=true to log out from all devices.

    Args:
        all_sessions (bool | Unset): If true, log out from all sessions (all devices) Default:
            False.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[HTTPValidationError | LogoutResponse]
     """


    kwargs = _get_kwargs(
        all_sessions=all_sessions,

    )

    response = await client.get_async_httpx_client().request(
        **kwargs
    )

    return _build_response(client=client, response=response)

async def asyncio(
    *,
    client: AuthenticatedClient | Client,
    all_sessions: bool | Unset = False,

) -> HTTPValidationError | LogoutResponse | None:
    """ Logout

     Log out the current user. Pass ?all_sessions=true to log out from all devices.

    Args:
        all_sessions (bool | Unset): If true, log out from all sessions (all devices) Default:
            False.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        HTTPValidationError | LogoutResponse
     """


    return (await asyncio_detailed(
        client=client,
all_sessions=all_sessions,

    )).parsed



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..types import UNSET, Unset






T = TypeVar("T", bound="AgentsUserConfig")



@_attrs_define
class AgentsUserConfig:
    """ 
        Attributes:
            enabled (bool | Unset): Whether to use agents mode for queries. Default: True.
            memory_enabled (bool | Unset): Run memory synthesis (skills, memories) after agent queries. Default: False.
            persona (str | Unset): Agent persona prepended to the system prompt. Customise to change the agent's identity
                and tone. Default: 'You are ARBI, an AI assistant created by ARBI CITY. Be proactive, helpful and
                professional.'.
            agent_model_name (str | Unset): The name of the model to be used for the agent decision-making. Default: 'auto'.
     """

    enabled: bool | Unset = True
    memory_enabled: bool | Unset = False
    persona: str | Unset = 'You are ARBI, an AI assistant created by ARBI CITY. Be proactive, helpful and professional.'
    agent_model_name: str | Unset = 'auto'
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)





    def to_dict(self) -> dict[str, Any]:
        enabled = self.enabled

        memory_enabled = self.memory_enabled

        persona = self.persona

        agent_model_name = self.agent_model_name


        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
        })
        if enabled is not UNSET:
            field_dict["ENABLED"] = enabled
        if memory_enabled is not UNSET:
            field_dict["MEMORY_ENABLED"] = memory_enabled
        if persona is not UNSET:
            field_dict["PERSONA"] = persona
        if agent_model_name is not UNSET:
            field_dict["AGENT_MODEL_NAME"] = agent_model_name

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        enabled = d.pop("ENABLED", UNSET)

        memory_enabled = d.pop("MEMORY_ENABLED", UNSET)

        persona = d.pop("PERSONA", UNSET)

        agent_model_name = d.pop("AGENT_MODEL_NAME", UNSET)

        agents_user_config = cls(
            enabled=enabled,
            memory_enabled=memory_enabled,
            persona=persona,
            agent_model_name=agent_model_name,
        )


        agents_user_config.additional_properties = d
        return agents_user_config

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..types import UNSET, Unset
from typing import cast

if TYPE_CHECKING:
  from ..models.skipped_file import SkippedFile





T = TypeVar("T", bound="UploadDocumentsResponse")



@_attrs_define
class UploadDocumentsResponse:
    """ Response for document upload operations.

    Returns array of document IDs for successfully uploaded files.
    Files that fail validation are skipped and listed with reasons.

        Attributes:
            doc_ext_ids (list[str] | Unset):
            batch_id (None | str | Unset): Batch ID for tracking completion via WebSocket batch_complete events
            skipped (list[SkippedFile] | Unset): Files that were skipped during upload with reasons
     """

    doc_ext_ids: list[str] | Unset = UNSET
    batch_id: None | str | Unset = UNSET
    skipped: list[SkippedFile] | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)





    def to_dict(self) -> dict[str, Any]:
        from ..models.skipped_file import SkippedFile
        doc_ext_ids: list[str] | Unset = UNSET
        if not isinstance(self.doc_ext_ids, Unset):
            doc_ext_ids = self.doc_ext_ids



        batch_id: None | str | Unset
        if isinstance(self.batch_id, Unset):
            batch_id = UNSET
        else:
            batch_id = self.batch_id

        skipped: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.skipped, Unset):
            skipped = []
            for skipped_item_data in self.skipped:
                skipped_item = skipped_item_data.to_dict()
                skipped.append(skipped_item)




        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
        })
        if doc_ext_ids is not UNSET:
            field_dict["doc_ext_ids"] = doc_ext_ids
        if batch_id is not UNSET:
            field_dict["batch_id"] = batch_id
        if skipped is not UNSET:
            field_dict["skipped"] = skipped

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.skipped_file import SkippedFile
        d = dict(src_dict)
        doc_ext_ids = cast(list[str], d.pop("doc_ext_ids", UNSET))


        def _parse_batch_id(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        batch_id = _parse_batch_id(d.pop("batch_id", UNSET))


        _skipped = d.pop("skipped", UNSET)
        skipped: list[SkippedFile] | Unset = UNSET
        if _skipped is not UNSET:
            skipped = []
            for skipped_item_data in _skipped:
                skipped_item = SkippedFile.from_dict(skipped_item_data)



                skipped.append(skipped_item)


        upload_documents_response = cls(
            doc_ext_ids=doc_ext_ids,
            batch_id=batch_id,
            skipped=skipped,
        )


        upload_documents_response.additional_properties = d
        return upload_documents_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..types import UNSET, Unset






T = TypeVar("T", bound="ProjectUpgradeRequest")



@_attrs_define
class ProjectUpgradeRequest:
    """ Change pack quantity on an existing project.

        Attributes:
            price_id (str):
            quantity (int):
            proration_behavior (str | Unset):  Default: 'create_prorations'.
     """

    price_id: str
    quantity: int
    proration_behavior: str | Unset = 'create_prorations'





    def to_dict(self) -> dict[str, Any]:
        price_id = self.price_id

        quantity = self.quantity

        proration_behavior = self.proration_behavior


        field_dict: dict[str, Any] = {}

        field_dict.update({
            "price_id": price_id,
            "quantity": quantity,
        })
        if proration_behavior is not UNSET:
            field_dict["proration_behavior"] = proration_behavior

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        price_id = d.pop("price_id")

        quantity = d.pop("quantity")

        proration_behavior = d.pop("proration_behavior", UNSET)

        project_upgrade_request = cls(
            price_id=price_id,
            quantity=quantity,
            proration_behavior=proration_behavior,
        )

        return project_upgrade_request




from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..types import UNSET, Unset
from typing import cast






T = TypeVar("T", bound="ProjectCheckoutRequest")



@_attrs_define
class ProjectCheckoutRequest:
    """ Create a Stripe checkout session for this project.

        Attributes:
            price_id (str):
            quantity (int | Unset): Number of Pro Packs Default: 1.
            promo_code (None | str | Unset): Promo code to auto-apply
     """

    price_id: str
    quantity: int | Unset = 1
    promo_code: None | str | Unset = UNSET





    def to_dict(self) -> dict[str, Any]:
        price_id = self.price_id

        quantity = self.quantity

        promo_code: None | str | Unset
        if isinstance(self.promo_code, Unset):
            promo_code = UNSET
        else:
            promo_code = self.promo_code


        field_dict: dict[str, Any] = {}

        field_dict.update({
            "price_id": price_id,
        })
        if quantity is not UNSET:
            field_dict["quantity"] = quantity
        if promo_code is not UNSET:
            field_dict["promo_code"] = promo_code

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        price_id = d.pop("price_id")

        quantity = d.pop("quantity", UNSET)

        def _parse_promo_code(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        promo_code = _parse_promo_code(d.pop("promo_code", UNSET))


        project_checkout_request = cls(
            price_id=price_id,
            quantity=quantity,
            promo_code=promo_code,
        )

        return project_checkout_request




from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset







T = TypeVar("T", bound="ProjectCreateRequest")



@_attrs_define
class ProjectCreateRequest:
    """ Create a new project.

        Attributes:
            name (str):
     """

    name: str





    def to_dict(self) -> dict[str, Any]:
        name = self.name


        field_dict: dict[str, Any] = {}

        field_dict.update({
            "name": name,
        })

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        name = d.pop("name")

        project_create_request = cls(
            name=name,
        )

        return project_create_request




from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..types import UNSET, Unset
from typing import cast

if TYPE_CHECKING:
  from ..models.price_tier import PriceTier





T = TypeVar("T", bound="ProductPrice")



@_attrs_define
class ProductPrice:
    """ Stripe product price information.

        Attributes:
            price_id (str):
            currency (str):
            interval (str):
            interval_count (int):
            default_price (bool):
            amount (int | None | Unset):
            billing_scheme (str | Unset):  Default: 'per_unit'.
            tiers (list[PriceTier] | None | Unset):
     """

    price_id: str
    currency: str
    interval: str
    interval_count: int
    default_price: bool
    amount: int | None | Unset = UNSET
    billing_scheme: str | Unset = 'per_unit'
    tiers: list[PriceTier] | None | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)





    def to_dict(self) -> dict[str, Any]:
        from ..models.price_tier import PriceTier
        price_id = self.price_id

        currency = self.currency

        interval = self.interval

        interval_count = self.interval_count

        default_price = self.default_price

        amount: int | None | Unset
        if isinstance(self.amount, Unset):
            amount = UNSET
        else:
            amount = self.amount

        billing_scheme = self.billing_scheme

        tiers: list[dict[str, Any]] | None | Unset
        if isinstance(self.tiers, Unset):
            tiers = UNSET
        elif isinstance(self.tiers, list):
            tiers = []
            for tiers_type_0_item_data in self.tiers:
                tiers_type_0_item = tiers_type_0_item_data.to_dict()
                tiers.append(tiers_type_0_item)


        else:
            tiers = self.tiers


        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "price_id": price_id,
            "currency": currency,
            "interval": interval,
            "interval_count": interval_count,
            "default_price": default_price,
        })
        if amount is not UNSET:
            field_dict["amount"] = amount
        if billing_scheme is not UNSET:
            field_dict["billing_scheme"] = billing_scheme
        if tiers is not UNSET:
            field_dict["tiers"] = tiers

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.price_tier import PriceTier
        d = dict(src_dict)
        price_id = d.pop("price_id")

        currency = d.pop("currency")

        interval = d.pop("interval")

        interval_count = d.pop("interval_count")

        default_price = d.pop("default_price")

        def _parse_amount(data: object) -> int | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(int | None | Unset, data)

        amount = _parse_amount(d.pop("amount", UNSET))


        billing_scheme = d.pop("billing_scheme", UNSET)

        def _parse_tiers(data: object) -> list[PriceTier] | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                tiers_type_0 = []
                _tiers_type_0 = data
                for tiers_type_0_item_data in (_tiers_type_0):
                    tiers_type_0_item = PriceTier.from_dict(tiers_type_0_item_data)



                    tiers_type_0.append(tiers_type_0_item)

                return tiers_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(list[PriceTier] | None | Unset, data)

        tiers = _parse_tiers(d.pop("tiers", UNSET))


        product_price = cls(
            price_id=price_id,
            currency=currency,
            interval=interval,
            interval_count=interval_count,
            default_price=default_price,
            amount=amount,
            billing_scheme=billing_scheme,
            tiers=tiers,
        )


        product_price.additional_properties = d
        return product_price

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..types import UNSET, Unset
from dateutil.parser import isoparse
from typing import cast
import datetime

if TYPE_CHECKING:
  from ..models.project_quotas import ProjectQuotas





T = TypeVar("T", bound="ProjectResponse")



@_attrs_define
class ProjectResponse:
    """ Project summary returned in list and detail views.

        Attributes:
            external_id (str):
            name (str):
            packs (int):
            subscription (str):
            created_at (datetime.datetime):
            quotas (ProjectQuotas): All quota metrics for a project, derived from packs.
            price_id (None | str | Unset):
            plan (None | str | Unset):
            amount (int | None | Unset):
            currency (None | str | Unset):
            current_period_end (int | None | Unset):
            cancel_at_period_end (bool | None | Unset):
            portal_url (None | str | Unset):
     """

    external_id: str
    name: str
    packs: int
    subscription: str
    created_at: datetime.datetime
    quotas: ProjectQuotas
    price_id: None | str | Unset = UNSET
    plan: None | str | Unset = UNSET
    amount: int | None | Unset = UNSET
    currency: None | str | Unset = UNSET
    current_period_end: int | None | Unset = UNSET
    cancel_at_period_end: bool | None | Unset = UNSET
    portal_url: None | str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)





    def to_dict(self) -> dict[str, Any]:
        from ..models.project_quotas import ProjectQuotas
        external_id = self.external_id

        name = self.name

        packs = self.packs

        subscription = self.subscription

        created_at = self.created_at.isoformat()

        quotas = self.quotas.to_dict()

        price_id: None | str | Unset
        if isinstance(self.price_id, Unset):
            price_id = UNSET
        else:
            price_id = self.price_id

        plan: None | str | Unset
        if isinstance(self.plan, Unset):
            plan = UNSET
        else:
            plan = self.plan

        amount: int | None | Unset
        if isinstance(self.amount, Unset):
            amount = UNSET
        else:
            amount = self.amount

        currency: None | str | Unset
        if isinstance(self.currency, Unset):
            currency = UNSET
        else:
            currency = self.currency

        current_period_end: int | None | Unset
        if isinstance(self.current_period_end, Unset):
            current_period_end = UNSET
        else:
            current_period_end = self.current_period_end

        cancel_at_period_end: bool | None | Unset
        if isinstance(self.cancel_at_period_end, Unset):
            cancel_at_period_end = UNSET
        else:
            cancel_at_period_end = self.cancel_at_period_end

        portal_url: None | str | Unset
        if isinstance(self.portal_url, Unset):
            portal_url = UNSET
        else:
            portal_url = self.portal_url


        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "external_id": external_id,
            "name": name,
            "packs": packs,
            "subscription": subscription,
            "created_at": created_at,
            "quotas": quotas,
        })
        if price_id is not UNSET:
            field_dict["price_id"] = price_id
        if plan is not UNSET:
            field_dict["plan"] = plan
        if amount is not UNSET:
            field_dict["amount"] = amount
        if currency is not UNSET:
            field_dict["currency"] = currency
        if current_period_end is not UNSET:
            field_dict["current_period_end"] = current_period_end
        if cancel_at_period_end is not UNSET:
            field_dict["cancel_at_period_end"] = cancel_at_period_end
        if portal_url is not UNSET:
            field_dict["portal_url"] = portal_url

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.project_quotas import ProjectQuotas
        d = dict(src_dict)
        external_id = d.pop("external_id")

        name = d.pop("name")

        packs = d.pop("packs")

        subscription = d.pop("subscription")

        created_at = isoparse(d.pop("created_at"))




        quotas = ProjectQuotas.from_dict(d.pop("quotas"))




        def _parse_price_id(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        price_id = _parse_price_id(d.pop("price_id", UNSET))


        def _parse_plan(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        plan = _parse_plan(d.pop("plan", UNSET))


        def _parse_amount(data: object) -> int | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(int | None | Unset, data)

        amount = _parse_amount(d.pop("amount", UNSET))


        def _parse_currency(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        currency = _parse_currency(d.pop("currency", UNSET))


        def _parse_current_period_end(data: object) -> int | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(int | None | Unset, data)

        current_period_end = _parse_current_period_end(d.pop("current_period_end", UNSET))


        def _parse_cancel_at_period_end(data: object) -> bool | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(bool | None | Unset, data)

        cancel_at_period_end = _parse_cancel_at_period_end(d.pop("cancel_at_period_end", UNSET))


        def _parse_portal_url(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        portal_url = _parse_portal_url(d.pop("portal_url", UNSET))


        project_response = cls(
            external_id=external_id,
            name=name,
            packs=packs,
            subscription=subscription,
            created_at=created_at,
            quotas=quotas,
            price_id=price_id,
            plan=plan,
            amount=amount,
            currency=currency,
            current_period_end=current_period_end,
            cancel_at_period_end=cancel_at_period_end,
            portal_url=portal_url,
        )


        project_response.additional_properties = d
        return project_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..types import UNSET, Unset
from typing import cast

if TYPE_CHECKING:
  from ..models.tag_template import TagTemplate





T = TypeVar("T", bound="DoctagLLMUserConfig")



@_attrs_define
class DoctagLLMUserConfig:
    r""" 
        Attributes:
            auto_rename (bool | Unset): Generate a descriptive document title via LLM instead of using the filename. The
                generated title replaces the filename-derived title in doc_metadata. Default: False.
            auto_rename_instruction (str | Unset): Instruction for generating document titles when AUTO_RENAME is enabled.
                Default: 'Create a short substantive title for the document of no more than 10 words. E.g. \\"Email from X to Y
                re ABC\\", \\"Service Agreement between A and B\\".'.
            default_metadata_tags (list[TagTemplate] | Unset): Metadata templates used for automatic document metadata
                extraction during indexing. Built-in tags cannot be removed; additional tags may be appended.
     """

    auto_rename: bool | Unset = False
    auto_rename_instruction: str | Unset = 'Create a short substantive title for the document of no more than 10 words. E.g. \\"Email from X to Y re ABC\\", \\"Service Agreement between A and B\\".'
    default_metadata_tags: list[TagTemplate] | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)





    def to_dict(self) -> dict[str, Any]:
        from ..models.tag_template import TagTemplate
        auto_rename = self.auto_rename

        auto_rename_instruction = self.auto_rename_instruction

        default_metadata_tags: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.default_metadata_tags, Unset):
            default_metadata_tags = []
            for default_metadata_tags_item_data in self.default_metadata_tags:
                default_metadata_tags_item = default_metadata_tags_item_data.to_dict()
                default_metadata_tags.append(default_metadata_tags_item)




        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
        })
        if auto_rename is not UNSET:
            field_dict["AUTO_RENAME"] = auto_rename
        if auto_rename_instruction is not UNSET:
            field_dict["AUTO_RENAME_INSTRUCTION"] = auto_rename_instruction
        if default_metadata_tags is not UNSET:
            field_dict["DEFAULT_METADATA_TAGS"] = default_metadata_tags

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.tag_template import TagTemplate
        d = dict(src_dict)
        auto_rename = d.pop("AUTO_RENAME", UNSET)

        auto_rename_instruction = d.pop("AUTO_RENAME_INSTRUCTION", UNSET)

        _default_metadata_tags = d.pop("DEFAULT_METADATA_TAGS", UNSET)
        default_metadata_tags: list[TagTemplate] | Unset = UNSET
        if _default_metadata_tags is not UNSET:
            default_metadata_tags = []
            for default_metadata_tags_item_data in _default_metadata_tags:
                default_metadata_tags_item = TagTemplate.from_dict(default_metadata_tags_item_data)



                default_metadata_tags.append(default_metadata_tags_item)


        doctag_llm_user_config = cls(
            auto_rename=auto_rename,
            auto_rename_instruction=auto_rename_instruction,
            default_metadata_tags=default_metadata_tags,
        )


        doctag_llm_user_config.additional_properties = d
        return doctag_llm_user_config

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..types import UNSET, Unset






T = TypeVar("T", bound="ModelUsage")



@_attrs_define
class ModelUsage:
    """ Usage metrics for a single model.

        Attributes:
            spend (float | Unset):  Default: 0.0.
            prompt_tokens (int | Unset):  Default: 0.
            completion_tokens (int | Unset):  Default: 0.
            total_tokens (int | Unset):  Default: 0.
            successful_requests (int | Unset):  Default: 0.
            failed_requests (int | Unset):  Default: 0.
     """

    spend: float | Unset = 0.0
    prompt_tokens: int | Unset = 0
    completion_tokens: int | Unset = 0
    total_tokens: int | Unset = 0
    successful_requests: int | Unset = 0
    failed_requests: int | Unset = 0
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)





    def to_dict(self) -> dict[str, Any]:
        spend = self.spend

        prompt_tokens = self.prompt_tokens

        completion_tokens = self.completion_tokens

        total_tokens = self.total_tokens

        successful_requests = self.successful_requests

        failed_requests = self.failed_requests


        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
        })
        if spend is not UNSET:
            field_dict["spend"] = spend
        if prompt_tokens is not UNSET:
            field_dict["prompt_tokens"] = prompt_tokens
        if completion_tokens is not UNSET:
            field_dict["completion_tokens"] = completion_tokens
        if total_tokens is not UNSET:
            field_dict["total_tokens"] = total_tokens
        if successful_requests is not UNSET:
            field_dict["successful_requests"] = successful_requests
        if failed_requests is not UNSET:
            field_dict["failed_requests"] = failed_requests

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        spend = d.pop("spend", UNSET)

        prompt_tokens = d.pop("prompt_tokens", UNSET)

        completion_tokens = d.pop("completion_tokens", UNSET)

        total_tokens = d.pop("total_tokens", UNSET)

        successful_requests = d.pop("successful_requests", UNSET)

        failed_requests = d.pop("failed_requests", UNSET)

        model_usage = cls(
            spend=spend,
            prompt_tokens=prompt_tokens,
            completion_tokens=completion_tokens,
            total_tokens=total_tokens,
            successful_requests=successful_requests,
            failed_requests=failed_requests,
        )


        model_usage.additional_properties = d
        return model_usage

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import cast

if TYPE_CHECKING:
  from ..models.agents_user_config import AgentsUserConfig
  from ..models.doctag_llm_user_config import DoctagLLMUserConfig
  from ..models.parser_user_config import ParserUserConfig





T = TypeVar("T", bound="NonDeveloperConfig")



@_attrs_define
class NonDeveloperConfig:
    """ 
        Attributes:
            agents (AgentsUserConfig):
            doctag_llm (DoctagLLMUserConfig):
            parser (ParserUserConfig):
     """

    agents: AgentsUserConfig
    doctag_llm: DoctagLLMUserConfig
    parser: ParserUserConfig
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)





    def to_dict(self) -> dict[str, Any]:
        from ..models.parser_user_config import ParserUserConfig
        from ..models.doctag_llm_user_config import DoctagLLMUserConfig
        from ..models.agents_user_config import AgentsUserConfig
        agents = self.agents.to_dict()

        doctag_llm = self.doctag_llm.to_dict()

        parser = self.parser.to_dict()


        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "Agents": agents,
            "DoctagLLM": doctag_llm,
            "Parser": parser,
        })

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.agents_user_config import AgentsUserConfig
        from ..models.doctag_llm_user_config import DoctagLLMUserConfig
        from ..models.parser_user_config import ParserUserConfig
        d = dict(src_dict)
        agents = AgentsUserConfig.from_dict(d.pop("Agents"))




        doctag_llm = DoctagLLMUserConfig.from_dict(d.pop("DoctagLLM"))




        parser = ParserUserConfig.from_dict(d.pop("Parser"))




        non_developer_config = cls(
            agents=agents,
            doctag_llm=doctag_llm,
            parser=parser,
        )


        non_developer_config.additional_properties = d
        return non_developer_config

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..types import UNSET, Unset
from typing import cast






T = TypeVar("T", bound="ProjectInvoice")



@_attrs_define
class ProjectInvoice:
    """ Single Stripe invoice/receipt for a project.

        Attributes:
            date (str):
            amount (int):
            currency (str):
            status (str):
            url (None | str | Unset):
            pdf_url (None | str | Unset):
     """

    date: str
    amount: int
    currency: str
    status: str
    url: None | str | Unset = UNSET
    pdf_url: None | str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)





    def to_dict(self) -> dict[str, Any]:
        date = self.date

        amount = self.amount

        currency = self.currency

        status = self.status

        url: None | str | Unset
        if isinstance(self.url, Unset):
            url = UNSET
        else:
            url = self.url

        pdf_url: None | str | Unset
        if isinstance(self.pdf_url, Unset):
            pdf_url = UNSET
        else:
            pdf_url = self.pdf_url


        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "date": date,
            "amount": amount,
            "currency": currency,
            "status": status,
        })
        if url is not UNSET:
            field_dict["url"] = url
        if pdf_url is not UNSET:
            field_dict["pdf_url"] = pdf_url

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        date = d.pop("date")

        amount = d.pop("amount")

        currency = d.pop("currency")

        status = d.pop("status")

        def _parse_url(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        url = _parse_url(d.pop("url", UNSET))


        def _parse_pdf_url(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        pdf_url = _parse_pdf_url(d.pop("pdf_url", UNSET))


        project_invoice = cls(
            date=date,
            amount=amount,
            currency=currency,
            status=status,
            url=url,
            pdf_url=pdf_url,
        )


        project_invoice.additional_properties = d
        return project_invoice

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import cast

if TYPE_CHECKING:
  from ..models.quota_usage import QuotaUsage





T = TypeVar("T", bound="ProjectQuotas")



@_attrs_define
class ProjectQuotas:
    """ All quota metrics for a project, derived from packs.

        Attributes:
            storage_gb (QuotaUsage): Single quota metric: current usage vs limit.
            ai_credits (QuotaUsage): Single quota metric: current usage vs limit.
            collaborators (QuotaUsage): Single quota metric: current usage vs limit.
     """

    storage_gb: QuotaUsage
    ai_credits: QuotaUsage
    collaborators: QuotaUsage
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)





    def to_dict(self) -> dict[str, Any]:
        from ..models.quota_usage import QuotaUsage
        storage_gb = self.storage_gb.to_dict()

        ai_credits = self.ai_credits.to_dict()

        collaborators = self.collaborators.to_dict()


        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "storage_gb": storage_gb,
            "ai_credits": ai_credits,
            "collaborators": collaborators,
        })

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.quota_usage import QuotaUsage
        d = dict(src_dict)
        storage_gb = QuotaUsage.from_dict(d.pop("storage_gb"))




        ai_credits = QuotaUsage.from_dict(d.pop("ai_credits"))




        collaborators = QuotaUsage.from_dict(d.pop("collaborators"))




        project_quotas = cls(
            storage_gb=storage_gb,
            ai_credits=ai_credits,
            collaborators=collaborators,
        )


        project_quotas.additional_properties = d
        return project_quotas

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..types import UNSET, Unset
from typing import cast

if TYPE_CHECKING:
  from ..models.project_invoice import ProjectInvoice





T = TypeVar("T", bound="ProjectInvoicesResponse")



@_attrs_define
class ProjectInvoicesResponse:
    """ Invoices for a project subscription.

        Attributes:
            project_ext_id (str):
            invoices (list[ProjectInvoice] | Unset):
     """

    project_ext_id: str
    invoices: list[ProjectInvoice] | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)





    def to_dict(self) -> dict[str, Any]:
        from ..models.project_invoice import ProjectInvoice
        project_ext_id = self.project_ext_id

        invoices: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.invoices, Unset):
            invoices = []
            for invoices_item_data in self.invoices:
                invoices_item = invoices_item_data.to_dict()
                invoices.append(invoices_item)




        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "project_ext_id": project_ext_id,
        })
        if invoices is not UNSET:
            field_dict["invoices"] = invoices

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.project_invoice import ProjectInvoice
        d = dict(src_dict)
        project_ext_id = d.pop("project_ext_id")

        _invoices = d.pop("invoices", UNSET)
        invoices: list[ProjectInvoice] | Unset = UNSET
        if _invoices is not UNSET:
            invoices = []
            for invoices_item_data in _invoices:
                invoices_item = ProjectInvoice.from_dict(invoices_item_data)



                invoices.append(invoices_item)


        project_invoices_response = cls(
            project_ext_id=project_ext_id,
            invoices=invoices,
        )


        project_invoices_response.additional_properties = d
        return project_invoices_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, BinaryIO, TextIO, TYPE_CHECKING, Generator

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..types import UNSET, Unset
from typing import cast

if TYPE_CHECKING:
  from ..models.daily_usage_model_groups import DailyUsageModelGroups





T = TypeVar("T", bound="DailyUsage")



@_attrs_define
class DailyUsage:
    """ Usage for a single day.

        Attributes:
            date (str):
            spend (float | Unset):  Default: 0.0.
            total_tokens (int | Unset):  Default: 0.
            api_requests (int | Unset):  Default: 0.
            model_groups (DailyUsageModelGroups | Unset):
     """

    date: str
    spend: float | Unset = 0.0
    total_tokens: int | Unset = 0
    api_requests: int | Unset = 0
    model_groups: DailyUsageModelGroups | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)





    def to_dict(self) -> dict[str, Any]:
        from ..models.daily_usage_model_groups import DailyUsageModelGroups
        date = self.date

        spend = self.spend

        total_tokens = self.total_tokens

        api_requests = self.api_requests

        model_groups: dict[str, Any] | Unset = UNSET
        if not isinstance(self.model_groups, Unset):
            model_groups = self.model_groups.to_dict()


        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "date": date,
        })
        if spend is not UNSET:
            field_dict["spend"] = spend
        if total_tokens is not UNSET:
            field_dict["total_tokens"] = total_tokens
        if api_requests is not UNSET:
            field_dict["api_requests"] = api_requests
        if model_groups is not UNSET:
            field_dict["model_groups"] = model_groups

        return field_dict



    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.daily_usage_model_groups import DailyUsageModelGroups
        d = dict(src_dict)
        date = d.pop("date")

        spend = d.pop("spend", UNSET)

        total_tokens = d.pop("total_tokens", UNSET)

        api_requests = d.pop("api_requests", UNSET)

        _model_groups = d.pop("model_groups", UNSET)
        model_groups: DailyUsageModelGroups | Unset
        if isinstance(_model_groups,  Unset):
            model_groups = UNSET
        else:
            model_groups = DailyUsageModelGroups.from_dict(_model_groups)




        daily_usage = cls(
            date=date,
            spend=spend,
            total_tokens=total_tokens,
            api_requests=api_requests,
            model_groups=model_groups,
        )


        daily_usage.additional_properties = d
        return daily_usage

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
